import java.util.*;
import java.text.SimpleDateFormat;
import java.applet.Applet;
import netscape.javascript.JSObject;
import netscape.javascript.JSException;


/**
* Cookies
*
*
* The cookies class represent a persistent set of properties. <br>
* The properties can be saved or loading by cookies on the page<br>
* Each key is a String. The value is a string with a date associated<br>
*
* @author C Caissotti, http://java.arcadevillage.com
*
* @see Hashtable
* @see Properties
*
* @since JDK 1.1
*
*/

public class Cookies extends Hashtable
{

/**
*
* Default number of day to keep a cookie
*
*/
 public int dd = 30;

 protected class CookieValue
 {
    String value;
    Calendar exp;
 }
/**
*
* Creates an empty cookies list with no default values.
*
*/

 public Cookies()
 {
}

/**
*
* Creates an empty cookies list with a default number of day to keep the cookies.
* @param dd - the time in days to keep the cookie ( default is 30 ) .
*
*/

 public Cookies( int dd )
 {
    super();
    this.dd = dd;
 }
/**
*
* Load the cookies list associated to the document where is the given applet.
* @param a - the applet.
*
*
*/

 public synchronized void load( Applet a )
 {
	String key;
	CookieValue cv;
	//
	// Now, we search all the cookies. Each cookies is separated by a ';'
	//
	int is = 0;
	int i = 0;
	try
	{
	JSObject win = JSObject.getWindow(a);
	JSObject doc = (JSObject)win.getMember("document");
	String s = ( String )doc.getMember("cookie" );
	while ( is < s.length() )
	{
	    // First we seach a =
	    cv = new CookieValue();
	    i = s.indexOf("=", is);
	    if ( i == -1 )
	      {
	        System.out.println("Error in cookie string :"+s);
	        return;
	      }
	    key = s.substring(is, i);
	    is = ++i;
	    // Now we search a ;
	    i = s.indexOf(";", is);
	    if ( i == -1 )
	      {
	        cv.value = s.substring(is);
	        is = s.length();
	      }
	    else
	      {
	        cv.value = s.substring(is,i);
	        is = i + 1; // Because the return string add a blank;
	        if ( s.charAt(is)==' ' ) is++;
	      }
	    put( key, cv );
	}
	}
	catch( Exception e )
	{
	    System.out.println("Load :"+e);
	}
 }
/**
*
* Returns an enumeration of all the keys in this property list, including the keys in the default property list.
* @return : an enumeration of all the keys in this property list, including the keys in the default property list.
*
* @see Enumeration
*
*/

 public Enumeration propertyNames()
 {
    return this.keys();
 }

/**
*
* Save the cookies list associated to the document where is the given applet.
* @param a - the applet.
*
*
*/
 public synchronized void save( Applet a )
 {
	String s = "";
	String sdate;
	String k;
	CookieValue cv;
    SimpleDateFormat df = new SimpleDateFormat("EEE, d MMM yyyy hh:mm:ss", Locale.US);
	//
	// We take all the properties and create a cookie string
	//
	Enumeration e = this.keys();
	while ( e.hasMoreElements() )
	{
	    k = ( String )e.nextElement();
	    cv = ( CookieValue )get(k);
	    if ( s.length() != 0 ) s = s + ";";
	    sdate = cv.exp == null ? "" : "+'; expires="+df.format(cv.exp.getTime())+" UTC'";

	    s = s + "'"+k+"='+escape('"+cv.value+"')"+sdate;
	}
	try
	{
	JSObject win = JSObject.getWindow(a);
    s = "document.cookie="+s+";";
	win.eval(s);
	}
	catch( Exception ex )
	{
	    System.out.println("Cookie : "+ex);
	}


 }

/**
*
* Searches for the property with the specified key in this property list. If the key is not found in this property list, the default property list, and its defaults, recursively, are then checked. The method returns null if the property is not found.
* @param key - the property key.
* @return the value in this property list with the specified key value.
*
*/

 public String getProperty(String key)
 {
  CookieValue v = ( CookieValue )this.get(key);
  return( v == null ? null : v.value );
 }

/**
*
* Searches for the property with the specified key in this property list. <br>
* If the key is not found in this property list, the default property list, and its defaults, recursively, are then checked. <br>
* The method returns the default value argument if the property is not found. <br>
* Parameters:
*
* @param key - the hashtable key.
* @param defaultvalue - a default value.
*
* @return the value in this property list with the specified key value.
*
*/

 public String getProperty(String key,
                          String defaultValue)
 {
  String s = getProperty(key);
  return( s == null ? defaultValue : s );
 }

/**
*
* Put the property with the specified key in the property list.<br>
* The expiration date of the property is the default expiration date
*
* @param key - the hashtable key.
* @param val - the value.
*
*
*/
public void setProperty(String key, String val )
 {
  setProperty(key, val, dd );
 }

/**
*
* Put the property with the specified key in the property list with the given
* expiration date
*
* @param key - the hashtable key.
* @param val - the value.
* @param nday - the number of day to keep the cookie.
*
*
*/
 public void setProperty(String key, String val, int nday )
 {
  CookieValue v = new CookieValue();
  v.value = val;
  v.exp = Calendar.getInstance();
  v.exp.add( Calendar.DAY_OF_MONTH, nday );
  put( key, v );
 }
}