import java.awt.*;

/**
*
* Class Sprite
*
* @author C Caissotti
*
*/

public abstract class Sprite
{
    private String name;
    public boolean opaque = true; // Does the other sprite can pass over this sprite
    public float x=0;
    public float y=0;
    public float basex = 0; // Pas de base x, utilis lorsque la vitesse d'un sprite ne varie jamais
    public float basey = 0;// Pas de base y, utilis lorsque la vitesse d'un sprite ne varie jamais
    public float pasx = 0;
    public float pasy = 0;
    public Point pos = new Point(0,0); // Contient la position en pixel du sprite
    public Point next = new Point(0,0); // Contiennent les prochaines positions de x et y
    public int idle = 1; // The environement has a idle incremented by 1 each time. The sprite move only when its idle modulo arrive
    public Rectangle p = new Rectangle(); // Polygone reprsentant le sprite
    boolean bmove; // Indique si le sprite pourra effectuer sont mouvement la prochaine fois
    public int poids; // Le poids peut tre utilis par exemple dans les problmes de collision
    public Dimension dim = null; // Dimension du sprite
    public int border=0;

    public Sprite( String s )
    {
        name = s;
}

    public void setPosition( int x, int y )
    {
        setPosition( ( float )x, ( float )y );
    }

    public void setPosition( float x, float y )
    {
        this.x = x;
        this.y = y;
        pos.setLocation(( int )x, ( int )y);
        next.setLocation(( int )x, ( int )y);
        if ( dim != null ) updateRectangle(pos, dim);
    }

    public void setDimension( Dimension d )
    {
        this.dim = new Dimension(d);
    }

    public void setDimension( int cx, int cy )
    {
        this.dim = new Dimension(cx,cy);
        updateRectangle(pos, dim);
    }

    public Dimension getDimension()
    {
        return dim;
    }


    public void init() // Called by the Game arcade at the begginning of each level
    {
    }

    public String getName()
    {
        return name;
    }

    public boolean touch( Sprite s )
    {
        return this.p.intersects( s.p );
    }


    public void prepareMove( int idle )
    {
        bmove = false;
        if ( pasx == 0 && pasy == 0 ) return;
        next.setLocation(( int )(x+pasx), ( int )(y+pasy));
        updateRectangle(next,dim);
        bmove = true;
    }

    public void move()
    {
        if ( bmove )
        {
         x += pasx;
         y += pasy;
         pos.setLocation(next);
         updateRectangle(pos,dim);
        }
    }

    public void updateRectangle( Point ref, Dimension d )
    {
      p.setBounds(ref.x+border,next.y+border, d.width-border*2,d.height-border*2);
    }

    /**
    *
    * This function must be implemented by the descendants of Sprite
    * It's called by the arcade motor before trying to move
    * pasx and pasy are modified by this function
    *
    */
    public abstract void think();

    /**
    *
    * This function must be implemented by the descendants of Sprite
    * It's called by the arcade motor when it paint the sprite
    *
    */
    public abstract void draw( Graphics g );

    public String toString()
     {
      return name+", "+pos+", "+dim;
     }

}